# frozen_string_literal: true

require_dependency "ugrp/ugrp_controller"

module Ugrp
  class AclMappingsController < UgrpController
    before_action :check_js, only: [:acl_change]

    # GET /ugrp/index
    def index
      respond_to do |format|
        format.html
        format.js do
          @domain = Ugrp::Domain.find(params[:domain_id])
          render layout: false
        end
      end
    end

    # GET /ugrp/users
    def users
      @users = if is_developer_or_super_user?
                 Ugrp::User.all
               else
                 current_ugrp_domain.users
               end

      respond_to do |format|
        format.html
        format.js do
          @domain = Ugrp::Domain.find(params[:domain_id])
          render layout: false
        end
      end
    end

    # GET /ugrp/domains
    def domains
      @domains = is_developer_or_super_user? ? Ugrp::Domain.all : current_ugrp_domain # + current_ugrp_domain.domain_aliases
    end

    # GET /ugrp/groups
    def groups
      @groups = if is_developer_or_super_user?
                  Ugrp::Group.all
                else
                  current_ugrp_domain.groups
                end

      respond_to do |format|
        format.html
        format.js do
          @domain = Ugrp::Domain.find(params[:domain_id])
          render layout: false
        end
      end
    end

    # GET /ugrp/roles
    def roles
      @roles = if is_developer_or_super_user?
                 Ugrp::Role.all
               else
                 current_ugrp_domain.roles
               end

      respond_to do |format|
        format.html
        format.js do
          @domain = Ugrp::Domain.find(params[:domain_id])
          render layout: false
        end
      end
    end

    # GET /ugrp/permissions
    def permissions
      @permissions = if is_developer_or_super_user?
                       Ugrp::Permission.all
                     else
                       current_ugrp_domain.permissions
                     end

      respond_to do |format|
        format.html
        format.js do
          @domain = Ugrp::Domain.find(params[:domain_id])
          render layout: false
        end
      end
    end

    # GET /ugrp/acl_mappings
    def acl_mappings
      if is_developer_or_super_user?
        @users = Ugrp::User.all
        @groups = Ugrp::Group.all
        @roles = Ugrp::Role.all
        @permissions = Ugrp::Permission.all
      else
        @users = current_ugrp_domain.users
        @groups = current_ugrp_domain.groups
        @roles = current_ugrp_domain.roles
        @permissions = current_ugrp_domain.permissions
      end

      respond_to do |format|
        format.html
        format.js do
          @domain = Ugrp::Domain.find(params[:domain_id])
          render layout: false
        end
      end
    end

    # POST /ugrp/acl_change/:acl/:state
    def acl_change
      errors = []

      acl = params[:acl]
      acl_name = (acl.split('-')[0]).to_s

      if %w[user_group user_role group_role group_permission role_permission].include? acl_name
        if %w[0 false 1 true].include? params[:state]

          value1 = acl.split('-')[1]
          value2 = acl.split('-')[2]

          state = %w[1 true].include? params[:state]

          # Validate/Map paremeters
          case acl_name
          when 'user_group'
            klass = Ugrp::UserGroup
            attribute1 = :user_id
            attribute2 = :group_id
          when 'user_role'
            klass = Ugrp::UserRole
            attribute1 = :user_id
            attribute2 = :role_id
          when 'group_role'
            klass = Ugrp::GroupRole
            attribute1 = :group_id
            attribute2 = :role_id
          when 'group_permission'
            klass = Ugrp::GroupPermission
            attribute1 = :group_id
            attribute2 = :permission_id
          when 'role_permission'
            klass = Ugrp::RolePermission
            attribute1 = :role_id
            attribute2 = :permission_id
          end

          # Perform the update
          begin
            if integer?(value1) && integer?(value2)
              if %w[user_group user_role group_role].include? acl_name
                if state
                  instance = klass.find_or_create_by(attribute1 => value1, attribute2 => value2)
                  if instance.nil?
                    errors << "Unable to perform update (#{acl_name})"
                  else
                    instance.save
                  end
                else
                  klass.where(attribute1 => value1, attribute2 => value2).delete_all
                end
              elsif %w[group_permission role_permission].include? acl_name
                operation = acl.split('-')[3]
                if %w[c r u d].include? operation
                  instance = klass.find_or_create_by(attribute1 => value1, attribute2 => value2)
                  if instance.nil?
                    errors << "Unable to perform update (#{acl_name})"
                  else
                    case operation
                    when 'c'
                      instance.permission.can_create = state
                    when 'r'
                      instance.permission.can_read = state
                    when 'u'
                      instance.permission.can_update = state
                    when 'd'
                      instance.permission.can_delete = state
                    end
                    instance.save
                    instance.permission.save
                  end
                else
                  errors << 'Invalid operation'
                end
              else
                errors << 'Unable to process ACL'
              end
            elsif !integer?(value1) && integer?(value2)
              errors << 'Unsupported parameters'
            elsif integer?(value1) && !integer?(value2)
              if %w[group_permission role_permission].include? acl_name
                operation = acl.split('-')[3]
                if %w[c r u d].include? operation

                  case acl_name
                  when 'group_permission'
                    group = Ugrp::Group.find(value1.to_i)
                    object_name = value2
                    name = "p_#{object_name}_#{group.name.gsub(/\s+/, '-')}"
                    descriptive_name = "Permissions on '#{object_name.capitalize}' for '#{group.name}'"

                    instance = Ugrp::GroupPermission
                               .joins(:permission)
                               .where(group: group, ugrp_permissions: { domain_id: current_ugrp_domain.id,
                                                                        object_name: object_name })
                               .first

                    if instance.nil?
                      instance = Ugrp::GroupPermission.create(
                        group: group,
                        permission: Ugrp::Permission.create(
                          domain: current_ugrp_domain,
                          name: name,
                          descriptive_name: descriptive_name,
                          object_name: object_name
                        )
                      )
                    end
                  when 'role_permission'
                    role = Ugrp::Role.find(value1.to_i)
                    object_name = value2
                    name = "p_#{object_name}_#{role.name.gsub(/\s+/, '-')}"
                    descriptive_name = "Permissions on '#{object_name.capitalize}' for '#{role.name}'"

                    instance = Ugrp::RolePermission
                               .joins(:permission)
                               .where(group: group, ugrp_permissions: { domain_id: current_ugrp_domain.id,
                                                                        object_name: object_name })
                               .first

                    if instance.nil?
                      instance = Ugrp::RolePermission.create(
                        role: role,
                        permission: Ugrp::Permission.create(
                          domain: current_ugrp_domain,
                          name: name,
                          descriptive_name: descriptive_name,
                          object_name: object_name
                        )
                      )
                    end
                  end

                  if instance.nil?
                    errors << "Unable to perform update (#{acl_name})"
                  else
                    case operation
                    when 'c'
                      instance.permission.can_create = state
                    when 'r'
                      instance.permission.can_read = state
                    when 'u'
                      instance.permission.can_update = state
                    when 'd'
                      instance.permission.can_delete = state
                    end
                    instance.save
                    instance.permission.save
                  end
                else
                  errors << 'Invalid operation'
                end
              else
                errors << 'Unable to process ACL'
              end
            else
              errors << 'Unprocessable ACL IDs'
            end
          rescue ActiveRecord::RecordNotUnique
            errors << 'RecordNotUnique'
            # retry
          end

          unless instance.nil?
            instance.errors.messages.each do |message|
              errors << message
            end
          end
        else
          errors << 'Invalid state'
        end
      else
        errors << "Invalid ACL (#{acl_name})"
      end

      respond_to do |format|
        format.json do
          if instance.nil?
            if errors.empty?
              render json: errors, status: :ok
            else
              render json: errors, status: :unprocessable_entity
            end
          elsif instance.errors.messages.empty?
            render json: instance, status: :ok
          else
            render json: instance.errors, status: :unprocessable_entity
          end
        end
      end
    end

    private

      def check_js
        raise RoutingError, 'expected application/json' unless request.format == :json
      end

      def integer?(object)
        object.to_i.to_s == object
      end

      def numeric?(object)
        object.to_i.to_s == object || object.to_f.to_s == object
      end
  end
end
